'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports['default'] = Dimensions;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

function _inherits(subClass, superClass) { if (typeof superClass !== 'function' && superClass !== null) { throw new TypeError('Super expression must either be null or a function, not ' + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) subClass.__proto__ = superClass; }

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var style = {
  width: '100%',
  height: '100%',
  padding: 0,
  border: 0
};

function defaultGetWidth(element) {
  return element.clientWidth;
}

function defaultGetHeight(element) {
  return element.clientHeight;
}

/**
 * Wraps a react component and adds properties `containerHeight` and
 * `containerWidth`. Useful for responsive design. Properties update on
 * window resize. **Note** that the parent element must have a height, or else
 * nothing will be rendered.
 *
 * Can be used as an [ES7 class decorator](https://github.com/wycats/javascript-decorators) or a
 * [higher-order component](http://babeljs.io/blog/2015/06/07/react-on-es6-plus/#property-initializers)
 * (see examples)
 *
 * @param  {function} [options.getHeight] `getHeight(element)` should return element
 * height, where element is the wrapper div. Defaults to `element.clientHeight`
 * @param  {function} [options.getWidth]  `getWidth(element)` should return element
 * width, where element is the wrapper div. Defaults to `element.clientWidth`
 * @return {function}                   Returns a decorator that can be used to
 * enhance a react component `Enhance(MyComponent)`
 * @example
 * import Dimensions from 'react-dimensions'
 *
 * class MyComponent {
 *   render() (
 *     <div>
 *       {`containerWidth=${this.props.containerWidth},`}
 *       {`containerHeight=${$this.props.containerHeight}`}
 *     </div>
 *   )
 * }
 *
 * export default Dimensions()(MyComponent) // Enhanced component
 */

function Dimensions() {
  var _ref = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

  var _ref$getHeight = _ref.getHeight;
  var getHeight = _ref$getHeight === undefined ? defaultGetHeight : _ref$getHeight;
  var _ref$getWidth = _ref.getWidth;
  var getWidth = _ref$getWidth === undefined ? defaultGetWidth : _ref$getWidth;

  return function (ComposedComponent) {
    return (function (_React$Component) {
      _inherits(_class, _React$Component);

      function _class() {
        var _this = this;

        _classCallCheck(this, _class);

        _React$Component.apply(this, arguments);

        this.state = {};

        this.updateDimensions = function () {
          var container = _react2['default'].findDOMNode(_this.refs.container);
          if (!container) {
            throw new Error('Cannot find container div');
          }
          _this.setState({
            containerWidth: getWidth(container),
            containerHeight: getHeight(container)
          });
        };

        this.onResize = function () {
          if (_this.rqf) return;
          _this.rqf = window.requestAnimationFrame(function () {
            _this.rqf = null;
            _this.updateDimensions();
          });
        };
      }

      _class.prototype.componentDidMount = function componentDidMount() {
        this.updateDimensions();
        window.addEventListener('resize', this.onResize, false);
      };

      _class.prototype.componentWillUnmount = function componentWillUnmount() {
        window.removeEventListener('resize', this.onResize);
      };

      _class.prototype.render = function render() {
        return _react2['default'].createElement(
          'div',
          { style: style, ref: 'container' },
          this.state.containerHeight && _react2['default'].createElement(ComposedComponent, _extends({}, this.state, this.props))
        );
      };

      return _class;
    })(_react2['default'].Component);
  };
}

module.exports = exports['default'];

// ES7 Class properties
// http://babeljs.io/blog/2015/06/07/react-on-es6-plus/#property-initializers

// Using arrow functions and ES7 Class properties to autobind
// http://babeljs.io/blog/2015/06/07/react-on-es6-plus/#arrow-functions
